#!/usr/bin/env python
# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

from ionas.request import Request, Entry, EntryList, XmlConfNode
from product_conf import SUDO_CMD
from fal_info import Results

from . import _utils

from fal_info import FAL_CLOUD_PATH


__empty = object()


def _call_load():
    entry = Entry('load', """<cloud />""")
    entry = _utils.send_entry(FAL_CLOUD_PATH, entry)

    cloud_node = entry.get()

    data = {}

    data['dropbox'] = []
    for dropbox_node in cloud_node.get_node_list('dropboxlist/dropbox'):
        data['dropbox'].append({
            'id': dropbox_node.get_value('id'),
            'name': dropbox_node.get_value('name'),
        })

    data['amazons3'] = []
    for amazons3_node in cloud_node.get_node_list('amazons3list/amazons3'):
        data['amazons3'].append({
            'id': amazons3_node.get_value('id'),
            'name': amazons3_node.get_value('name'),
            'accesskey': amazons3_node.get_value('accesskey'),
            'secretkey': amazons3_node.get_value('secretkey'),
            'httpmethod': amazons3_node.get_value('httpmethod'),
            'storageclass': amazons3_node.get_value('storageclass'),
            'endpoint': amazons3_node.get_value('endpoint'),
            'port': amazons3_node.get_value('port'),
            'mpupload': amazons3_node.get_node('mpupload').is_enabled(),
        })

    data['cloudn'] = []
    for cloudn_node in cloud_node.get_node_list('cloudnlist/cloudn'):
        data['cloudn'].append({
            'id': cloudn_node.get_value('id'),
            'name': cloudn_node.get_value('name'),
            'accesskey': cloudn_node.get_value('accesskey'),
            'secretkey': cloudn_node.get_value('secretkey'),
            'httpmethod': cloudn_node.get_value('httpmethod'),
            'storageclass': cloudn_node.get_value('storageclass'),
            'endpoint': cloudn_node.get_value('endpoint'),
            'port': cloudn_node.get_value('port'),
            'mpupload': cloudn_node.get_node('mpupload').is_enabled(),
        })

    data['nifcloud'] = []
    for nifcloud_node in cloud_node.get_node_list('nifcloudlist/nifcloud'):
        data['nifcloud'].append({
            'id': nifcloud_node.get_value('id'),
            'name': nifcloud_node.get_value('name'),
            'accesskey': nifcloud_node.get_value('accesskey'),
            'secretkey': nifcloud_node.get_value('secretkey'),
            'httpmethod': nifcloud_node.get_value('httpmethod'),
            'storageclass': nifcloud_node.get_value('storageclass'),
            'endpoint': nifcloud_node.get_value('endpoint'),
            'port': nifcloud_node.get_value('port'),
            'mpupload': nifcloud_node.get_node('mpupload').is_enabled(),
        })

    data['azure'] = []
    for azure_node in cloud_node.get_node_list('azurelist/azure'):
        data['azure'].append({
            'id': azure_node.get_value('id'),
            'name': azure_node.get_value('name'),
            'accountname': azure_node.get_value('accountname'),
            'accesskey': azure_node.get_value('accesskey'),
            'httpmethod': azure_node.get_value('httpmethod'),
        })

    data['box'] = []
    for box_node in cloud_node.get_node_list('boxlist/box'):
        data['box'].append({
            'id': box_node.get_value('id'),
            'name': box_node.get_value('name'),
        })

    data['onedrive'] = []
    for onedrive_node in cloud_node.get_node_list('onedrivelist/onedrive'):
        data['onedrive'].append({
            'id': onedrive_node.get_value('id'),
            'name': onedrive_node.get_value('name'),
        })

    data['onedrivebusiness'] = []
    for onedrivebusiness_node in cloud_node.get_node_list('onedrivebusinesslist/onedrivebusiness'):
        data['onedrivebusiness'].append({
            'id': onedrivebusiness_node.get_value('id'),
            'name': onedrivebusiness_node.get_value('name'),
        })

    data['googledrive'] = []
    for googledrive_node in cloud_node.get_node_list('googledrivelist/googledrive'):
        data['googledrive'].append({
            'id': googledrive_node.get_value('id'),
            'name': googledrive_node.get_value('name'),
        })

    data['fletsazukeru'] = []
    for azukeru_node in cloud_node.get_node_list('fletsazukerulist/fletsazukeru'):
        data['fletsazukeru'].append({
            'id': azukeru_node.get_value('id'),
            'name': azukeru_node.get_value('name'),
            'contractor': azukeru_node.get_value('contractor'),
            'plan': azukeru_node.get_value('plan'),
            'authmethod': azukeru_node.get_value('authmethod'),
            'otherauthmethod': azukeru_node.get_value('otherauthmethod'),
            'terminalkey': azukeru_node.get_value('terminalkey'),
            'clientid': azukeru_node.get_value('clientid'),
            'password': azukeru_node.get_value('password'),
        })

    data['mdcloud'] = []
    for nifcloud_node in cloud_node.get_node_list('mdcloudlist/mdcloud'):
        data['mdcloud'].append({
            'id': nifcloud_node.get_value('id'),
            'name': nifcloud_node.get_value('name'),
            'accesskey': nifcloud_node.get_value('accesskey'),
            'secretkey': nifcloud_node.get_value('secretkey'),
            'httpmethod': nifcloud_node.get_value('httpmethod'),
            'storageclass': nifcloud_node.get_value('storageclass'),
            'endpoint': nifcloud_node.get_value('endpoint'),
            'port': nifcloud_node.get_value('port'),
            'mpupload': nifcloud_node.get_node('mpupload').is_enabled(),
        })

    return data


def _call_add_or_modify(operation, spk, parameters, messages):
    entry = Entry(operation, """<cloud />""")
    cloud_node = entry.get()

    xxx_node = XmlConfNode(spk)
    if operation == 'modify':
        xxx_node.append_child('id', text=parameters['id'])

    if spk == 'googledrive':
        xxx_node.append_child('name', text=parameters['name'])
        if operation == 'add':
            xxx_node.append_child('code', text=parameters['code'])

    if spk == 'dropbox':
        xxx_node.append_child('name', text=parameters['name'])
        if operation == 'add':
            xxx_node.append_child('code', text=parameters['code'])

    if spk == 'box':
        xxx_node.append_child('name', text=parameters['name'])
        if operation == 'add':
            xxx_node.append_child('code', text=parameters['code'])

    if spk == 'amazons3':
        xxx_node.append_child('name', text=parameters['name'])
        xxx_node.append_child('accesskey', text=parameters['accesskey'])
        xxx_node.append_child('secretkey', text=parameters['secretkey'])
        xxx_node.append_child('httpmethod', text=parameters['httpmethod'])
        xxx_node.append_child('storageclass', text=parameters['storageclass'])
        xxx_node.append_child('endpoint', text=parameters['endpoint'])
        xxx_node.append_child('port', text=str(parameters['port']))
        xxx_node.append_child('mpupload', attrs={
            'enable': str(parameters['mpupload']).lower(),
        })

    if spk == 'cloudn':
        xxx_node.append_child('name', text=parameters['name'])
        xxx_node.append_child('accesskey', text=parameters['accesskey'])
        xxx_node.append_child('secretkey', text=parameters['secretkey'])
        xxx_node.append_child('httpmethod', text=parameters['httpmethod'])
        xxx_node.append_child('storageclass', text=parameters['storageclass'])
        xxx_node.append_child('endpoint', text=parameters['endpoint'])
        xxx_node.append_child('port', text=str(parameters['port']))
        xxx_node.append_child('mpupload', attrs={
            'enable': str(parameters['mpupload']).lower(),
        })

    if spk == 'nifcloud':
        xxx_node.append_child('name', text=parameters['name'])
        xxx_node.append_child('accesskey', text=parameters['accesskey'])
        xxx_node.append_child('secretkey', text=parameters['secretkey'])
        xxx_node.append_child('httpmethod', text=parameters['httpmethod'])
        xxx_node.append_child('storageclass', text=parameters['storageclass'])
        xxx_node.append_child('endpoint', text=parameters['endpoint'])
        xxx_node.append_child('port', text=str(parameters['port']))
        xxx_node.append_child('mpupload', attrs={
            'enable': str(parameters['mpupload']).lower(),
        })

    if spk == 'fletsazukeru':
        xxx_node.append_child('name', text=parameters['name'])
        xxx_node.append_child('contractor', text=parameters['contractor'])
        xxx_node.append_child('plan', text=parameters['plan'])
        xxx_node.append_child('authmethod', text=parameters['authmethod'])
        xxx_node.append_child('otherauthmethod',
                                text=parameters['otherauthmethod'])
        xxx_node.append_child('terminalkey',
				text=parameters['terminalkey'])
        xxx_node.append_child('clientid', text=parameters['clientid'])
        xxx_node.append_child('password', text=parameters['password'])

    if spk == 'azure':
        xxx_node.append_child('name', text=parameters['name'])
        xxx_node.append_child('accountname', text=parameters['accountname'])
        xxx_node.append_child('accesskey', text=parameters['accesskey'])
        xxx_node.append_child('httpmethod', text=parameters['httpmethod'])

    if spk == 'onedrive':
        xxx_node.append_child('name', text=parameters['name'])
        if operation == 'add':
            xxx_node.append_child('code', text=parameters['code'])

    if spk == 'onedrivebusiness':
        xxx_node.append_child('name', text=parameters['name'])
        if operation == 'add':
            xxx_node.append_child('code', text=parameters['code'])

    if spk == 'mdcloud':
        xxx_node.append_child('name', text=parameters['name'])
        xxx_node.append_child('accesskey', text=parameters['accesskey'])
        xxx_node.append_child('secretkey', text=parameters['secretkey'])
        xxx_node.append_child('httpmethod', text=parameters['httpmethod'])
        xxx_node.append_child('storageclass', text=parameters['storageclass'])
        xxx_node.append_child('endpoint', text=parameters['endpoint'])
        xxx_node.append_child('port', text=str(parameters['port']))
        xxx_node.append_child('mpupload', attrs={
            'enable': str(parameters['mpupload']).lower(),
        })

    cloud_node.append_child(xxx_node)

    entry = _utils.send_entry(FAL_CLOUD_PATH, entry)

    result = entry.get_result()

    constname, arglist = messages.get(result, messages['__else__'])
    message = _utils.gettext(constname, arglist=arglist)
    _utils.logging(constname, arglist)

    return {
        'is_succeeded': result in (Results.OK,),
        'message': message,
    }


def _call_add(spk, parameters):
    name = parameters['name']
    return _call_add_or_modify('add', spk, parameters, {
        Results.OK: ['MSG_CLOUD_ADD', { 'name': name }],
        '__else__': ['ERR_CLOUD_ADD', { 'name': name }],
    })


def _call_modify(spk, parameters):
    name = parameters['name']
    return _call_add_or_modify('modify', spk, parameters, {
        Results.OK: ['MSG_CLOUD_MODIFY', { 'name': name }],
        '__else__': ['ERR_CLOUD_MODIFY', { 'name': name }],
    })


def _call_delete(spk, profile_id):
    profile = obtain_profile(spk, profile_id)
    name = profile['name'] if profile else ''
    messages = {
        Results.OK: ['MSG_CLOUD_DELETE', { 'name': name }],
        '__else__': ['ERR_CLOUD_DELETE', { 'name': name }],
    }
    try:
        entry = Entry('delete', '<cloud />')
        cloud_node = entry.get()
        cloud_node.append_child(spk)
        xxx_node = cloud_node.get_node(spk)
        xxx_node.append_child('id', profile_id)
        entry = _utils.send_entry(FAL_CLOUD_PATH, entry)

        result = entry.get_result()

        constname, arglist = messages.get(result, messages['__else__'])
        message = _utils.gettext(constname, arglist=arglist)
        _utils.logging(constname, arglist)

        return {
            'is_succeeded': result in (Results.OK,),
            'message': message,
        }
    except:
        pass


def _call_check(spk, profile_id):
    # messages = {
    #     Results.OK: ['MSG_CLOUD_CHECK', {}],
    #     '__else__': ['ERR_CLOUD_CHECK', {}],
    # }
    try:
        entry = Entry('check', '<cloud />')
        cloud_node = entry.get()
        cloud_node.append_child(spk)
        xxx_node = cloud_node.get_node(spk)
        xxx_node.append_child('id', profile_id)
        entry = _utils.send_entry(FAL_CLOUD_PATH, entry)

        result = entry.get_result()

        # constname, arglist = messages.get(result, messages['__else__'])
        # message = _utils.gettext(constname, arglist=arglist)
        # _utils.logging(constname, arglist)

        return {
            'is_succeeded': result in (Results.OK,),
            # 'message': message,
        }
    except:
        return {
            'is_succeeded': False,
            'message': None,
        }


def __get_authorize_url(spk, default=__empty):
    try:
        entry = Entry('get', '<cloud />')
        entry.get().append_child(spk)
        entry = _utils.send_entry(FAL_CLOUD_PATH, entry)
        cloud_node = entry.get()
        url = cloud_node.get_node(spk).get_node('url').get_text()
        return url
    except:
        if default is __empty:
            raise
        return default


###############################################################################


def obtain_profiles(spk):
    data = _call_load()
    profiles = data.get(spk, [])
    return profiles


def obtain_profile(spk, profile_id):
    for profile in obtain_profiles(spk):
        if profile['id'] == profile_id:
            return profile
    return None


def create_profile(spk, parameters):
    return _call_add(spk, parameters)


def update_profile(spk, parameters):
    return _call_modify(spk, parameters)


def delete_profile(spk, profile_id):
    return _call_delete(spk, profile_id)


def check_profile(spk, profile_id):
    return _call_check(spk, profile_id)['is_succeeded']


def get_authorize_url_for_dropbox():
    return __get_authorize_url('dropbox')


def get_authorize_url_for_box():
    return __get_authorize_url('box')


def get_authorize_url_for_onedrive():
    return __get_authorize_url('onedrive')


def get_authorize_url_for_onedriveb():
    return __get_authorize_url('onedrivebusiness')


def get_authorize_url_for_googledrive():
    return __get_authorize_url('googledrive')
